/**********************************************************************

      Code for calculating the evolution of the Sivers function 
      from the Torino fits  (Eur.Phys. J. A39,89(2009))
      with evolution implemented following arXiv:1110.6428.


	  TorinoSivers(xval,ktval,qval,fl) is a callable 
         function WITH THE CORRECT NORMALIZATION that
              returns the Sivers function with:
                   xval = Bjorken-x
                   ktval = transvserse momentum (in GeV)
                   qval = hard scale (in GeV)
                   fl = flavor (1=up, 2=down)


	  The Sivers functions here are Gaussians obtained
      with parameters depending on q and parametrized 
      previously with the FindFit routine in 
      Wolfram Mathematica.         
	   

      The normalization of the Torino fits uses the 
      collinear pdfs.  In this case we link to 
	  the MSTW pdfs.
       

      
**********************************************************************/

#include <iostream>
#include "mstwpdf.h"
#include <math.h>
#include "sivers.h"

using namespace std;

c_mstwpdf *pdf = new c_mstwpdf("mstw2008lo.00.dat");


double up(double xa) {
return pdf->parton(2,xa,sqrt(2.4))/xa;
}

double down(double xa){
return pdf->parton(1,xa,sqrt(2.4))/xa;
}
  
  
double Nup (double xa){
double alpha = 0.73;
double beta = 3.46;
double N=0.35;

return N*pow(xa,alpha)*pow(1.0-xa,beta)*pow(alpha+beta,alpha+beta)/pow(alpha,alpha)/pow(beta,beta);

}

double Ndown (double xa){
double alpha = 1.08;
double beta = 3.46;
double N=0.90;

return N*pow(xa,alpha)*pow(1.0-xa,beta)*pow(alpha+beta,alpha+beta)/pow(alpha,alpha)/pow(beta,beta);

}


double fonepertup (double xa,double kt2zero) {

double Mp=0.938;
double M1=sqrt(0.34);
double kt2 = 0.25;

return -Mp*sqrt(2.0*exp(1.0))/M1/kt2*Nup(xa)*up(xa)*kt2zero;
}


double fonepertdown (double xa,double kt2zero) {

double Mp=0.938;
double M1=sqrt(0.34);
double kt2 = 0.25;

return -Mp*sqrt(2.0*exp(1.0))/M1/kt2*Ndown(xa)*down(xa)*kt2zero;
}


double gaussnormfitlowtorinoup (double q) {

return 0.856642/(-7.92053*q + 5.3952*q*q - 0.0307745*pow(q,4) - 0.00134312*pow(q,6));

}

double gaussnormfithightorinoup (double q) {

return 0.00020356 + 1635.84/(-47186.2*sqrt(q) + 43453.6*q);

}

double gaussparfittorinoup (double q) {

return  1.4556/log(0.513906*q*q);
}


double gaussnormfitlowtorinodown (double q) {

return 4.67378/(-32.0522*q + 21.8329*q*q - 0.124536*pow(q,4) - 0.00543525*pow(q,6));

}

double gaussnormfithightorinodown (double q) {

return 0.000274447 + 746.554 / (-15972.4*sqrt(q) + 14708.9*q);  
}

double gaussparfittorinodown (double q) {

return  1.4556/log(0.513906*q*q);

}




double TorinoUpSivers (double xa, double kt, double q) {


double xval = 0.1;
double Mp = 0.938;
double kt2=0.25;
double M1=sqrt(0.34);



double kt2zerovaltorino = M1*M1*kt2/(M1*M1 + kt2); 

double factor1 = -Mp/2/kt/xval;
double factor2 = fonepertup(xa,kt2zerovaltorino) / fonepertup(xval,kt2zerovaltorino);

if (q<=5) return factor1*factor2 * gaussnormfitlowtorinoup(q) * kt * exp(-gaussparfittorinoup(q)*kt*kt);
else if (q>5) return factor1*factor2 * gaussnormfithightorinoup(q) * kt * exp(-gaussparfittorinoup(q)*kt*kt);

}


double TorinoDownSivers (double xa, double kt, double q) {


double xval = 0.1;
double Mp = 0.938;
double kt2=0.25;
double M1=sqrt(0.34);



double kt2zerovaltorino = M1*M1*kt2/(M1*M1 + kt2); 

double factor1 = Mp/2/kt/xval;
double factor2 = fonepertdown(xa,kt2zerovaltorino) / fonepertdown(xval,kt2zerovaltorino);

if (q<=5) return factor1*factor2 * gaussnormfitlowtorinodown(q) * kt * exp(-gaussparfittorinodown(q)*kt*kt);
else if (q>5) return factor1*factor2 * gaussnormfithightorinodown(q) * kt * exp(-gaussparfittorinodown(q)*kt*kt);

}

double TorinoSivers(double xa, double kt, double q, int fl) {

if (fl==1) return TorinoUpSivers(xa,kt,q);
else if (fl==2) return TorinoDownSivers(xa,kt,q);
else {
	cout <<"Unrecognized Flavor" << endl;
	exit(1);
	}

}



